/*
 * file Location.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

/**
 * The location of a resource.
 * <p>
 * The format of the location string is specific to the repository that stores
 * the persistent resource.  A URL, a UNC filename, and an NFS filename
 * are examples of possible formats for a location string.  Locations are
 * constructed by the {@link Provider#location} method. </p>
 * <p>
 * All methods in this interface are implemented in-memory only, i.e.,
 * no access to the repository is required. </p>
 * 
 * @since 1.0
 */
public interface Location {

    /**
     * Return a string value for this Location.
     * The {@link Provider#location} method
     * is the inverse of this method, i.e., 
     * <code>location.equals(Provider.location(location.string()))</code>.
     * 
     * @return a string value for this Location.
     */
    public String string();

    /**
     * Return the location of a folder that has the resource at
     * this Location as a bound member whose binding name is
     * the lastSegment of this Location.
     * 
     * @return the location of a folder that has the resource at
     * this Location as a bound member whose binding name is
     * the lastSegment of this Location.
     * If this Location has no parent, <code>null</code> is returned.
     */
    public Location parent();

    /**
     * Return the location of the bound member with the specified
     * bindingName in the folder at this Location.
     * 
     * @param bindingName the name of the bound member.
     * The last segment of the returned location must be bindingName.
     * The bindingName commonly is not allowed to contain the "/" character.
     * @return the location of the bound member with the specified
     * binding name in the folder at this Location.
     * @throws WvcmException
     * if this Location is not one that can have bound members,
     * or if the bindingName is not syntactically valid.
     */
    public Location child(String bindingName) throws WvcmException;

    /**
     * Get the last segment of this Location.
     * If the {@link #parent} is <code>null</code>, the {@link #lastSegment} is <code>null</code>.
     * In general, if <code>loc</code> is a Location that has a parent,
     * <code>loc.parent().child(loc.lastSegment()).equals(loc)</code>.
     * 
     * @return the last segment of this Location.
     * Commonly, this is the portion following the last "/" in
     * the string value of this Location.
     */
    public String lastSegment();
}



